// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>

#include <Receiver_impl.h>

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <termio.h>
#include <sys/ioctl.h>

static struct termio TerminalSettings;

static void
ResetTerm(int sig)
{
    //
    // Reset terminal
    //
    ioctl(STDIN_FILENO, TCSETA, &TerminalSettings);

    //
    // Send signal again
    //
    signal(sig, SIG_DFL);
    kill(getpid(), sig);
}

int
run(CORBA_ORB_ptr orb, const char* host, CORBA_UShort portNum,
    int argc, char* argv[])
{
    //
    // Create BOA
    //
    CORBA_BOA_var boa = orb -> BOA_init(argc, argv);

    //
    // Get "server" object
    //
    CORBA_Object_var obj =
	orb -> get_inet_object(host, portNum, "Chat-Server");
    assert(!CORBA_is_nil(obj));

    Broadcaster_var server = Broadcaster::_narrow(obj);
    assert(!CORBA_is_nil(server));

    //
    // Create client object
    //
    Receiver_var client = new Receiver_impl(server);

    //
    // Run implementation
    //
    boa -> impl_is_ready(CORBA_ImplementationDef::_nil());

    //
    // Bye-bye
    //
    return 0;
}

int
main(int argc, char* argv[], char*[])
{
    //
    // Get host and port
    //
    char host[1024];
    char port[256];
    cout << "Host name? " << flush;
    cin >> host;
    cout << "Port number? " << flush;
    cin >> port;

    CORBA_UShort portNum = atoi(port);

    cout << "Using object `Chat-Server' from host `" << host
	 << "' on port " << portNum << "..." << endl;

    char c;
    cin.read(&c, 1);

    //
    // Set terminal mode
    //
    struct termio ctl;
    ioctl(STDIN_FILENO, TCGETA, &TerminalSettings);
    ioctl(STDIN_FILENO, TCGETA, &ctl);
    ctl.c_lflag &= ~(ICANON | ECHO);
    ctl.c_cc[VMIN] = 1;
    ctl.c_cc[VTIME] = 0;
    ioctl(STDIN_FILENO, TCSETA, &ctl);

    //
    // Set signal handler
    //
    signal(SIGHUP, ResetTerm);
    signal(SIGINT, ResetTerm);
    signal(SIGTERM, ResetTerm);
    signal(SIGQUIT, ResetTerm);

    int status;

    try
    {
	//
	// Create ORB
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);
	orb -> nested(true);

	//
	// Generate code
	//
	status = run(orb, host, portNum, argc, argv);
	
	//
	// Nothing should be printed here, since all objects should
	// have been released after the return of run()
	//
	orb -> print_active_objects(cout);
    }
#ifdef __GNUG__
    catch(CORBA_COMM_FAILURE& ex)
#else
    catch(CORBA_SystemException& ex)
#endif
    {
	OBPrintException(ex);
	status = 1;
    }

    //
    // Reset terminal
    //
    ioctl(STDIN_FILENO, TCSETA, &TerminalSettings);

    return status;
}
