// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>
#include <Broadcaster_impl.h>

#include <stdlib.h>
#include <time.h>

int
run(CORBA_ORB_ptr orb, int argc, char* argv[])
{
    //
    // Create BOA
    //
    CORBA_BOA_var boa = orb -> BOA_init(argc, argv);

    //
    // Create implementation object
    //
    Broadcaster_var p = new Broadcaster_impl("Chat-Server");

    //
    // Print "server started" message
    //
    time_t t;
    char* d;
    t = time(0);
    d = asctime(localtime(&t));
    assert(d && strlen(d) > 0);
    d[strlen(d) - 1] = '\0';
    cout << "*** Server started at " << d << endl;

    //
    // Print host and port
    //
    cout << "*** Running on host `" << boa -> host()
	 << "', port number " << boa -> port() << endl;

    //
    // Run implementation
    //
    boa -> impl_is_ready(CORBA_ImplementationDef::_nil());

    return 0;
}

int
main(int argc, char* argv[], char*[])
{
    int status;

    try
    {
	//
	// Create ORB
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);
	
	//
	// Generate code
	//
	status = run(orb, argc, argv);
	
	//
	// Nothing should be printed here, since all objects should
	// have been released after the return of run()
	//
	orb -> print_active_objects(cout);
    }
#ifdef __GNUG__
    catch(CORBA_COMM_FAILURE& ex)
#else
    catch(CORBA_SystemException& ex)
#endif
    {
	OBPrintException(ex);
	status = 1;
    }

    return status;
}
