// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package chat;

import java.awt.*;
import java.applet.*;
import org.omg.CORBA.*;

class ClientWindow extends Frame
{
    //
    // The receiver
    //
    private Receiver_impl receiver_;

    //
    // The "Widgets"
    //
    private TextField inputArea_;
    private TextArea outputArea_;
    private MenuItem quit_;
    private MenuItem info_;
    private MenuItem cmds_;

    //
    // Constructor
    //
    ClientWindow(Broadcaster server, BOA boa)
    {
	//
	// Create client window
	//
        MenuBar menubar = new MenuBar();

        Menu menuFile = new Menu("File", true);
        menubar.add(menuFile);
	quit_ = new MenuItem("Quit");
	menuFile.add(quit_);

	Menu menuHelp = new Menu("Help");
	menubar.add(menuHelp);
	info_ = new MenuItem("About Java-Chat...");
	menuHelp.add(info_);
	cmds_ = new MenuItem("Commands...");
	menuHelp.add(cmds_);
        menubar.setHelpMenu(menuHelp);

        setMenuBar(menubar);

	setFont(new Font("Courier", Font.PLAIN, 12));

        GridBagLayout l = new GridBagLayout();
	setLayout(l);

	GridBagConstraints c1 = new GridBagConstraints();
	outputArea_ = new TextArea(24, 80);
	outputArea_.setEditable(false);
	c1.fill = GridBagConstraints.BOTH;
	c1.gridx = 0;
	c1.gridy = 0;
	c1.gridwidth = 1;
	c1.gridheight = 1;
	c1.weightx = 1.0;
	c1.weighty = 1.0;
	add(outputArea_);
	l.setConstraints(outputArea_, c1);

	GridBagConstraints c2 = new GridBagConstraints();
	inputArea_ = new TextField(80);
	inputArea_.setEditable(true);
	c2.fill = GridBagConstraints.BOTH;
	c2.gridx = 0;
	c2.gridy = 1;
	c2.gridwidth = 1;
	c2.gridheight = 1;
	c2.weightx = 1.0;
	c2.weighty = 0.0;
	add(inputArea_);
	l.setConstraints(inputArea_, c2);

	pack();
        show();

	//
	// Create client object
	//
	receiver_ = new Receiver_impl(this, server, boa);
    }

    //
    // Event handling
    //
    public boolean
    action(Event e, java.lang.Object arg)
    {
	String cmd = arg.toString();

	if(cmd.equals(quit_.getLabel()))
	    receiver_.cmdQuit();
	else if(cmd.equals(info_.getLabel()))
	    receiver_.cmdAbout();
	else if(cmd.equals(cmds_.getLabel()))
	    receiver_.cmdHelp();
	else
	{
	    String input = inputArea_.getText();
	    
	    if(input.length() > 0)
		receiver_.parse(input);

	    inputArea_.setText("");
	}

	return true;
    }

    public boolean
    handleEvent(Event e)
    {
	if(e.id == e.WINDOW_DESTROY)
	    receiver_.cmdQuit();

	return super.handleEvent(e);
    }

    //
    // Print message
    //
    public void
    print(String text)
    {
	outputArea_.appendText(text);
	outputArea_.appendText("\n");
    }
}
