// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1997
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

package test;

import java.io.*;
import org.omg.CORBA.*;

public class Client extends TestBase
{
    static public int
    run(ORB orb, String args[])
	throws TestException
    {
	//
	// Get TestInterface
	//
	String impl;
	String implDSI;

	try
	{
	    String refFile = "TestInterface.ref";
	    FileInputStream file = new FileInputStream(refFile);
	    DataInputStream in = new DataInputStream(file);
	    impl = in.readLine();
	    implDSI = in.readLine();
	    file.close();
 	}
	catch(IOException ex)
	{
	    System.err.println("Can't read from `" +
			       ex.getMessage() + "'");
	    return 1;
	}

	//
	// Run tests
	//
	System.out.print("Testing constants... ");
	System.out.flush();
	new TestConst();
	System.out.println("OK!");

	System.out.print("Testing TypeCode type... ");
	System.out.flush();
	new TestTypeCode();
	System.out.println("OK!");

	System.out.print("Testing any type... ");
	System.out.flush();
	new TestAny();
	System.out.println("OK!");

	System.out.print("Testing some union types... ");
	System.out.flush();
	new TestUnion();
	System.out.println("OK!");

	System.out.print("Testing string_to_object()... ");
	System.out.flush();
 	org.omg.CORBA.Object obj = orb.string_to_object(impl);
 	TEST(obj != null);
	org.omg.CORBA.Object objDSI = orb.string_to_object(implDSI);
	TEST(objDSI != null);
	System.out.println("OK!");

	System.out.print("Testing _narrow()... ");
	System.out.flush();
 	TestInterface ti = TestInterfaceHelper.narrow(obj);
 	TEST(ti != null);
	TestInterface tiDSI = TestInterfaceHelper.narrow(objDSI);
	TEST(tiDSI != null);
 	TestInterfaceEx tiex = TestInterfaceExHelper.narrow(obj);
 	TEST(tiex != null);
	TestInterfaceEx tiexDSI = TestInterfaceExHelper.narrow(objDSI);
	TEST(tiexDSI != null);
 	TestInterfaceTimeout titimeout = TestInterfaceTimeoutHelper.narrow(ti);
 	TEST(titimeout != null);
	System.out.println("OK!");

 	System.out.print("Testing interface stub... ");
 	System.out.flush();
 	new TestStub(ti);
 	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStub(tiDSI);
	System.out.println("OK!");

	System.out.print("Testing interface stub with contexts... ");
	System.out.flush();
	new TestStubContext(ti);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStubContext(tiDSI);
	System.out.println("OK!");

	boolean gnu = false;
	String language = ti.language();
	if(language.equals("g++"))
	    gnu = true;

	if(gnu)
	{
	    System.out.println("WARNING:");
	    System.out.println(
		"At least one of the test programs was compiled with GNU C++."
		);
	    System.out.println(
		"Can't execute all tests due to buggy exception handling.");
	}

	System.out.print("Testing interface stub with exceptions... ");
	System.out.flush();
	new TestStubException(tiex, false, gnu);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestStubException(tiexDSI, true, gnu);
	System.out.println("OK!");

// 	System.out.print("Testing interface stub with timeouts... ");
// 	System.out.flush();
// 	new TestStubTimeout(titimeout);
// 	System.out.println("OK!");

	System.out.print("Testing DII... ");
	System.out.flush();
	new TestDII(obj);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDII(objDSI);
	System.out.println("OK!");

	System.out.print("Testing DII with contexts... ");
	System.out.flush();
	new TestDIIContext(obj);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDIIContext(tiDSI);
	System.out.println("OK!");

	System.out.print("Testing DII with exceptions... ");
	System.out.flush();
	new TestDIIException(obj);
	System.out.println("OK!");

	System.out.print("Ditto, but with the DSI implementation... ");
	System.out.flush();
	new TestDIIException(objDSI);
	System.out.println("OK!");

	ti.deactivate();

	return 0;
    }

    static public void
    main(String args[])
    {
	int status = 0;

        try
        {
            //
            // Create ORB
            //
            ORB orb = ORB.init(args, new java.util.Properties());
 
	    //
	    // Run tests
	    //
	    status = run(orb, args);
	}
	catch(SystemException ex)
        {
	    System.err.println(ex.getMessage());
	    ex.printStackTrace();
	    TEST(false);
	}

	//
	// Sleep for a short time so that the sender thread can
	// complete the transmission of the last oneway `deactivate' call
	//
	try
	{
	    Thread.currentThread().sleep(2000);
	}
	catch(InterruptedException ex)
	{
	}

	System.exit(status);
    }
}
