// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>
#include <Test_impl.h>
#include <Test2_skel_tie.h>
#include <TestDSI_impl.h>

#include <stdlib.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifndef HAVE_NO_EXPLICIT_TEMPLATES
template class TestInterfaceTimeout_skel_tie< TestInterfaceTimeout_impl >;
#else
#ifdef HAVE_PRAGMA_DEFINE
#pragma define(TestInterfaceTimeout_skel_tie< TestInterfaceTimeout_impl >)
#endif
#endif

int
run(CORBA_ORB_ptr orb, int argc, char* argv[])
{
    //
    // Create BOA
    //
    CORBA_BOA_var boa = orb -> BOA_init(argc, argv);
	
    //
    // Create implementation object
    //
    TestInterfaceTimeout_impl* i =
	new TestInterfaceTimeout_impl("test");
    TestInterfaceTimeout_var p = i;
    CORBA_String_var impl = orb -> object_to_string(p);

    TestInterfaceExDSI_impl* iDSI =
	new TestInterfaceExDSI_impl("testDSI", p, orb);
    CORBA_Object_var pDSI = iDSI;
    CORBA_String_var implDSI = orb -> object_to_string(pDSI);

    TestInterfaceTimeout_var pTie =
	new TestInterfaceTimeout_skel_tie< TestInterfaceTimeout_impl >
	("testTie", i, CORBA_FALSE);
    CORBA_String_var implTie = orb -> object_to_string(pTie);

    const char* refFile = "TestInterface.ref";
    ofstream out(refFile);
    if(out.fail())
    {
	extern int errno;
	cerr << argv[0] << ": can't open `" << refFile << "': "
	     << strerror(errno) << endl;
	return 1;
    }
    
    out << impl << endl;
    out << implDSI << endl;
    out << implTie << endl;
    out.close();
    
    boa -> impl_is_ready(CORBA_ImplementationDef::_nil());

    i -> releaseInternal();
    iDSI -> releaseInternal();
	
#ifdef HAVE__UNLINK
    _unlink(refFile);
#else
    unlink(refFile);
#endif

    return 0;
}


int
main(int argc, char* argv[], char*[])
{
    int status;

    try
    {
	//
	// Create ORB
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);
	
	//
	// Run tests
	//
	status = run(orb, argc, argv);

	//
	// Nothing should be printed here, since all objects should
	// have been released after the return of run()
	//
	orb -> print_active_objects(cerr);
    }
#ifdef __GNUG__
    catch(CORBA_COMM_FAILURE& ex)
#else
    catch(CORBA_SystemException& ex)
#endif
    {
	OBPrintException(ex);
	status = 1;
    }

    return status;
}
