// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_BOA_H
#define OB_BOA_H

//
// Declare classes
//
class CORBA_Object_skel;
class OBIIOPAcceptor;

class CORBA_Principal;
typedef CORBA_Principal* CORBA_Principal_ptr;

//
// The BOA class
//
class CORBA_BOA;
typedef CORBA_BOA* CORBA_BOA_ptr;
typedef CORBA_BOA_ptr BOARef;
typedef OBObjVar<CORBA_BOA> CORBA_BOA_var;

class CORBA_BOA : protected OBRefCount
{
    static CORBA_BOA_ptr boa_; // The BOA singleton

    //
    // Hide copy-constructor and asignment operator
    //
    CORBA_BOA(const CORBA_BOA&);
    void operator=(const CORBA_BOA&);

protected:

    CORBA_BOA();
    virtual ~CORBA_BOA();

public:

    //
    // Standard IDL to C++ Mapping
    //
    friend inline void CORBA_release(CORBA_BOA_ptr p)
    { if(p) p -> _OB_decRef(); }
    friend inline CORBA_Boolean CORBA_is_nil(CORBA_BOA_ptr p)
    { return p == 0; }
    static inline CORBA_BOA_ptr _duplicate(CORBA_BOA_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CORBA_BOA_ptr _nil()
    { return 0; }

    CORBA_Object_ptr create(const CORBA_ReferenceData&,
			    CORBA_InterfaceDef_ptr,
			    CORBA_ImplementationDef_ptr);

    void dispose(CORBA_Object_ptr);

    CORBA_ReferenceData* get_id(CORBA_Object_ptr);

    void change_implementation(CORBA_Object_ptr, CORBA_ImplementationDef_ptr);

    CORBA_Principal_ptr get_principal(CORBA_Object_ptr, CORBA_Environment_ptr);

    virtual void impl_is_ready(CORBA_ImplementationDef_ptr) = 0;
    virtual void deactivate_impl(CORBA_ImplementationDef_ptr) = 0;

    virtual void obj_is_ready(CORBA_Object_ptr,
			      CORBA_ImplementationDef_ptr) = 0;
    virtual void deactivate_obj(CORBA_Object_ptr) = 0;

    //
    // Additonal OmniBroker specific functions
    //
    virtual const char* host() const = 0;
    virtual CORBA_UShort port() const = 0;

    //
    // OmniBroker internal functions
    // Application programs must not use these functions directly
    //
    static CORBA_BOA_ptr _OB_instance();
    virtual CORBA_Object_ptr _OB_find(const OBObjectKey&);
    virtual OBDispatchStatus _OB_dispatch(const OBObjectKey&, const char*,
					  OBFixSeq< CORBA_Octet >&, bool,
					  CORBA_ULong, CORBA_ULong);
    virtual void _OB_writeProfiles(CORBA_IOP_IOR&, const OBObjectKey&) = 0;
};

//
// The Object skeleton class
//
class CORBA_Object_skel : public virtual CORBA_Object
{
    //
    // The Object skeleton object number
    //
    static CORBA_ULong _ob_num_;

    //
    // Sequence with all object names currently in use
    //
    static OBStrSeq _ob_names_;

    //
    // Hide copy-constructor and asignment operator
    //
    CORBA_Object_skel(const CORBA_Object_skel&);
    void operator=(const CORBA_Object_skel&);

protected:

    CORBA_Object_skel();
    virtual ~CORBA_Object_skel();

public:

    //
    // Standard IDL to C++ Mapping
    //
    CORBA_Object_ptr _this() { return CORBA_Object::_duplicate(this); }
    CORBA_BOA_ptr _boa() { return CORBA_BOA::_OB_instance(); }
    virtual CORBA_ImplementationDef_ptr _get_implementation();
    virtual CORBA_InterfaceDef_ptr _get_interface();
    virtual CORBA_Boolean _non_existent();

    //
    // Additonal OmniBroker specific functions
    //
    virtual CORBA_Boolean _is_dynamic();

    //
    // OmniBroker internal functions
    // Application programs must not use these functions directly
    //
    virtual const CORBA_IOP_IOR& _OB_ior();
    virtual const OBObjectKey& _OB_key();

    void _OB_createObjectKeyWithName(const char*);
    virtual CORBA_ULong _OB_incNumber() const;
    virtual OBDispatchStatus _OB_dispatch(const char*,
					  OBFixSeq< CORBA_Octet >&, bool,
					  CORBA_ULong, CORBA_ULong);
};

#endif
