// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_ORB_H
#define OB_ORB_H

//
// Declare classes
//
class CORBA_BOA;
typedef CORBA_BOA* CORBA_BOA_ptr;

class CORBA_Context;
typedef CORBA_Context* CORBA_Context_ptr;

class CORBA_Environment;
typedef CORBA_Environment* CORBA_Environment_ptr;

class CORBA_OperationDef;
typedef CORBA_OperationDef* CORBA_OperationDef_ptr;

class CORBA_DynAny;
typedef CORBA_DynAny* CORBA_DynAny_ptr;

class CORBA_DynStruct;
typedef CORBA_DynStruct* CORBA_DynStruct_ptr;

class CORBA_DynSequence;
typedef CORBA_DynSequence* CORBA_DynSequence_ptr;

class CORBA_DynArray;
typedef CORBA_DynArray* CORBA_DynArray_ptr;

class CORBA_DynUnion;
typedef CORBA_DynUnion* CORBA_DynUnion_ptr;

class CORBA_DynEnum;
typedef CORBA_DynEnum* CORBA_DynEnum_ptr;

//
// Plain object C++-Pointer sequence
//
typedef OBFixSeq< CORBA_Object* > OBObjectPtrSeq;

//
// Helper struct for initial services
//
struct OBInitialService
{
    CORBA_String_var name;
    CORBA_IOP_IOR ior;
};
typedef OBFixSeq< OBInitialService > OBInitialServiceSeq;

//
// The ORB class
//
class CORBA_ORB;
typedef CORBA_ORB* CORBA_ORB_ptr;
typedef CORBA_ORB_ptr ORBRef;
typedef OBObjVar<CORBA_ORB> CORBA_ORB_var;

class CORBA_ORB : protected OBRefCount
{
    static CORBA_ORB_ptr orb_; // The ORB singleton

    //
    // Hide copy-constructor and asignment operator
    //
    CORBA_ORB(const CORBA_ORB&);
    void operator=(const CORBA_ORB&);

protected:

    //
    // Note: The ORB should not contain any object references, Since
    // this would introduce cyclic object dependenices. Therefore IORs
    // are used here. This does also have the advantage, that it is
    // possible to completely release the following references within
    // the client code, saving memory and networking resources.
    //
    OBInitialServiceSeq services_; // List of initial services

    //
    // Plain C++ pointers to all registered objects
    //
    // A object reference sequence can not be used here, since then
    // the reference count of an object will never fall to zero and
    // therefore an object would never be deleted.
    //
    OBObjectPtrSeq objectSeq_;

    CORBA_ORB();
    virtual ~CORBA_ORB();

public:

    //
    // Standard IDL to C++ Mapping
    //
    friend inline void CORBA_release(CORBA_ORB_ptr p)
    { if(p) p -> _OB_decRef(); }
    friend inline CORBA_Boolean CORBA_is_nil(CORBA_ORB_ptr p)
    { return p == 0; }
    static inline CORBA_ORB_ptr _duplicate(CORBA_ORB_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CORBA_ORB_ptr _nil()
    { return 0; }

    typedef char* CORBA_OAid;
    CORBA_BOA_ptr BOA_init(int&, char**, const char* = "");

    virtual char* object_to_string(CORBA_Object_ptr) = 0;
    virtual CORBA_Object_ptr string_to_object(const char*) = 0;

    CORBA_Status create_list(CORBA_Long, CORBA_NVList_ptr&);
    CORBA_Status create_operation_list(CORBA_OperationDef_ptr,
				       CORBA_NVList_ptr&);
    CORBA_Status create_named_value(CORBA_NamedValue_ptr&);
    CORBA_Status create_exception_list(CORBA_ExceptionList_ptr&);
    CORBA_Status create_context_list(CORBA_ContextList_ptr&);

    CORBA_Status get_default_context(CORBA_Context_ptr&);
    CORBA_Status create_environment(CORBA_Environment_ptr&);

    typedef OBObjSeq<CORBA_Request> RequestSeq;
    CORBA_Status send_multiple_requests_oneway(const RequestSeq&);
    CORBA_Status send_multiple_requests_deferred(const RequestSeq&);
    CORBA_Boolean poll_next_response();
    CORBA_Status get_next_response(CORBA_Request_ptr&);

    static CORBA_TypeCode_ptr create_struct_tc(const char*,
					       const char*,
					       const CORBA_StructMemberSeq&);
    
    static CORBA_TypeCode_ptr create_union_tc(const char*,
					      const char*,
					      CORBA_TypeCode_ptr,
					      const CORBA_UnionMemberSeq&);
    
    static CORBA_TypeCode_ptr create_enum_tc(const char*,
					     const char*,
					     const CORBA_EnumMemberSeq&);
    
    static CORBA_TypeCode_ptr create_alias_tc(const char*,
					      const char*,
					      CORBA_TypeCode_ptr);
    
    static CORBA_TypeCode_ptr create_exception_tc(const char*,
						  const char*,
						 const CORBA_StructMemberSeq&);
    
    static CORBA_TypeCode_ptr create_interface_tc(const char*,
						  const char*);
    
    static CORBA_TypeCode_ptr create_string_tc(CORBA_ULong);
    
    static CORBA_TypeCode_ptr create_sequence_tc(CORBA_ULong,
						 CORBA_TypeCode_ptr);
    
    static CORBA_TypeCode_ptr create_recursive_sequence_tc(CORBA_ULong,
							   CORBA_ULong);
    
    static CORBA_TypeCode_ptr create_array_tc(CORBA_ULong,
					      CORBA_TypeCode_ptr);

    typedef char* ObjectId;
    typedef CORBA_String_var ObjectId_var;
    typedef OBStrSeq ObjectIdList;
    typedef OBSeqVar< OBStrSeq > ObjectIdList_var;

    class InvalidName { };

    virtual ObjectIdList* list_initial_services();
    virtual CORBA_Object_ptr resolve_initial_references(const char*);
        
    struct InconsistentTypeCode {};
    CORBA_DynAny_ptr create_dyn_any(const CORBA_Any& value);
    CORBA_DynAny_ptr create_basic_dyn_any(CORBA_TypeCode_ptr type);
    CORBA_DynStruct_ptr create_dyn_struct(CORBA_TypeCode_ptr type);
    CORBA_DynSequence_ptr create_dyn_sequence(CORBA_TypeCode_ptr type);
    CORBA_DynArray_ptr create_dyn_array(CORBA_TypeCode_ptr type);
    CORBA_DynUnion_ptr create_dyn_union(CORBA_TypeCode_ptr type);
    CORBA_DynEnum_ptr create_dyn_enum(CORBA_TypeCode_ptr type);

    //
    // Additional OmniBroker specific functions
    //
    void repository(CORBA_Object_ptr);
    void naming(CORBA_Object_ptr);
    void print_active_objects(ostream&);
    virtual bool nested() const = 0;
    virtual void nested(bool) = 0;
    virtual CORBA_Object_ptr get_inet_object(const char*, CORBA_UShort,
					     const char*) = 0;

    //
    // OmniBroker internal functions
    // Application programs must not use these functions directly
    //
    static CORBA_ORB_ptr _OB_instance();

    void _OB_register(CORBA_Object*);
    void _OB_unregister(CORBA_Object*);

    const OBObjectPtrSeq* _OB_getRegistered() const { return &objectSeq_; }

    CORBA_Object_ptr _OB_newObject(const CORBA_IOP_IOR&, const OBObjectKey&,
				   OBConnection_ptr);
    void _OB_setObject(CORBA_Object_ptr,
		       const CORBA_IOP_IOR&, const OBObjectKey&,
		       OBConnection_ptr);

    virtual CORBA_Object_ptr _OB_createObject(const CORBA_IOP_IOR&) = 0;
    virtual void _OB_changeObject(CORBA_Object_ptr, const CORBA_IOP_IOR&) = 0;

    virtual bool _OB_compareIORs(const CORBA_IOP_IOR&,
				 const CORBA_IOP_IOR&) const = 0;

    void _OB_addInitialService(const char*, const char*);
    void _OB_addInitialService(const char*, CORBA_Object_ptr);
    void _OB_parseConfigFile(const char*);
};

//
// ORB initialization
//
CORBA_ORB_ptr CORBA_ORB_init(int&, char**, const char* = "");
typedef char* CORBA_ORBid;

#endif
