// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OBJECT_H
#define OB_OBJECT_H

//
// Define object key
//
typedef OBFixSeq<CORBA_Octet> OBObjectKey;

//
// The OBConnection class
//
class OBConnection;
typedef OBConnection* OBConnection_ptr;
typedef OBConnection_ptr OBConnectionRef;
typedef OBObjVar< OBConnection > OBConnection_var;

class OBConnection : protected OBRefCount
{
    //
    // Hide copy-constructor and asignment operator
    //
    OBConnection(const OBConnection&);
    void operator=(const OBConnection&);

protected:

    OBConnection() { }
    virtual ~OBConnection() { }

public:

    friend inline void CORBA_release(OBConnection_ptr p)
    { if(p) p -> _OB_decRef(); }
    friend inline CORBA_Boolean CORBA_is_nil(OBConnection_ptr p)
    { return p == 0; }
    static inline OBConnection_ptr _duplicate(OBConnection_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline OBConnection_ptr _nil()
    { return 0; }

    virtual void* _OB_narrowHelp(const char*) const = 0;

    //
    // Calculate offset
    //
    virtual CORBA_ULong offset(CORBA_Object_ptr, const char*) = 0;

    //
    // Send request and receive response
    // With explicit timeout (throws exception on timeout)
    //
    virtual CORBA_ULong request(CORBA_Object_ptr, const char*,
				OBFixSeq< CORBA_Octet >&, bool&, bool&, bool&,
				CORBA_Long) = 0;

    //
    // Send request oneway
    // With implicit zero timeout (does not throw exception on timeout)
    //
    virtual void oneway(CORBA_Object_ptr, const char*,
			OBFixSeq< CORBA_Octet >&) = 0;

    //
    // Send request deferred
    // With implicit zero timeout (does not throw exception on timeout)
    //
    virtual void deferred(CORBA_Object_ptr, const char*,
			  OBFixSeq< CORBA_Octet >&, CORBA_ULong&) = 0;

    //
    // Get repsonse
    // With explicit timeout (throws exception on timeout)
    //
    virtual CORBA_ULong response(CORBA_ULong, OBFixSeq< CORBA_Octet >&,
				 bool&, bool&, bool&, CORBA_Long) = 0;
    
    //
    // Poll response
    // With implicit zero timeout (returns zero on timeout)
    //
    virtual CORBA_ULong poll(CORBA_ULong, OBFixSeq< CORBA_Octet >&,
			     bool&, bool&, bool&) = 0;
};

//
// Declare classes
//
class CORBA_Context;
typedef CORBA_Context* CORBA_Context_ptr;

class CORBA_NamedValue;
typedef CORBA_NamedValue* CORBA_NamedValue_ptr;

class CORBA_NVList;
typedef CORBA_NVList* CORBA_NVList_ptr;

class CORBA_ExceptionList;
typedef CORBA_ExceptionList* CORBA_ExceptionList_ptr;

class CORBA_ContextList;
typedef CORBA_ContextList* CORBA_ContextList_ptr;

class CORBA_Request;
typedef CORBA_Request* CORBA_Request_ptr;

class CORBA_InterfaceDef;
typedef CORBA_InterfaceDef* CORBA_InterfaceDef_ptr;

class CORBA_OperationDef;
typedef CORBA_OperationDef* CORBA_OperationDef_ptr;

class CORBA_ImplementationDef;
typedef CORBA_ImplementationDef* CORBA_ImplementationDef_ptr;

//
// The Dispatch Status
//
enum OBDispatchStatus
{
    OBDispatchStatusOK,
    OBDispatchStatusExcept,
    OBDispatchStatusNoObject,
    OBDispatchStatusNoOperation
};

//
// The Object class
//
class CORBA_Object;
typedef CORBA_Object* CORBA_Object_ptr;
typedef CORBA_Object_ptr CORBA_ObjectRef;
typedef OBObjVar< CORBA_Object > CORBA_Object_var;

class CORBA_Object : protected OBRefCount
{
    //
    // Hide copy-constructor and asignment operator
    //
    CORBA_Object(const CORBA_Object&);
    void operator=(const CORBA_Object&);

protected:

    //
    // The CORBA_IOP_IOR
    //
    CORBA_IOP_IOR _ob_ior_;

    //
    // The object key
    //
    OBObjectKey _ob_key_;

    //
    // The connection
    //
    OBConnection_var _ob_con_;

    //
    // The timeout (in msec)
    //
    CORBA_Long _ob_tout_;

    CORBA_Object();
    CORBA_Object(const CORBA_IOP_IOR&, const OBObjectKey&, OBConnection_ptr);
    virtual ~CORBA_Object();

    friend class CORBA_ORB; // The ORB may create CORBA_Objects

public:

    //
    // Standard IDL to C++ Mapping
    //
    friend void CORBA_release(CORBA_Object_ptr);
    friend CORBA_Boolean CORBA_is_nil(CORBA_Object_ptr);
    static inline CORBA_Object_ptr _duplicate(CORBA_Object_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CORBA_Object_ptr _nil()
    { return 0; }

    virtual CORBA_ImplementationDef_ptr _get_implementation();
    virtual CORBA_InterfaceDef_ptr _get_interface();

    virtual CORBA_Boolean _is_a(const char*);
    virtual CORBA_Boolean _non_existent();
    CORBA_Boolean _is_equivalent(CORBA_Object_ptr);
    CORBA_ULong _hash(CORBA_ULong);

    CORBA_Status _create_request(CORBA_Context_ptr, const char*,
				 CORBA_NVList_ptr, CORBA_NamedValue_ptr,
				 CORBA_Request_ptr&, CORBA_Flags);
    
    CORBA_Status _create_request(CORBA_Context_ptr, const char*,
				 CORBA_NVList_ptr, CORBA_NamedValue_ptr,
				 CORBA_ExceptionList_ptr,
				 CORBA_ContextList_ptr,
				 CORBA_Request_ptr&, CORBA_Flags);

    CORBA_Request_ptr _request(const char* operation);

    //
    // Additonal OmniBroker specific functions
    //
    CORBA_Boolean _is_local();
    virtual CORBA_Boolean _is_dynamic();
    CORBA_Long _timeout();
    void _timeout(CORBA_Long);

    //
    // OmniBroker internal functions
    // Application programs must not use these functions directly
    //
    virtual const CORBA_IOP_IOR& _OB_ior();
    virtual const OBObjectKey& _OB_key();
    OBConnection_ptr _OB_connection();

    virtual const char* _OB_typeId() const;
    virtual void* _OB_narrowHelp(const char*) const;
    virtual OBDispatchStatus _OB_dispatch(const char*,
					  OBFixSeq< CORBA_Octet >&,
					  bool swap, CORBA_ULong, CORBA_ULong);
    CORBA_Boolean _OB_remoteIsA(const char*);
    void _OB_copyFrom(CORBA_Object_ptr);
    void _OB_forward(const CORBA_Octet*&, bool);

    //
    // CDR
    //
    friend void OBMarshal(CORBA_Object_ptr, CORBA_Octet*&);
    friend void OBMarshalCount(CORBA_Object_ptr, CORBA_ULong&);
    friend void OBUnmarshal(CORBA_Object_ptr&, const CORBA_Octet*&, bool);
};

//
// release and is_nil
//
inline void
CORBA_release(CORBA_Object_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline CORBA_Boolean
CORBA_is_nil(CORBA_Object_ptr p)
{
    return p == 0;
}

//
// CDR
//
void OBMarshal(const OBObjSeq< CORBA_Object >&, CORBA_Octet*&);
void OBMarshalCount(const OBObjSeq< CORBA_Object >&, CORBA_ULong&);
void OBUnmarshal(OBObjSeq< CORBA_Object >&, const CORBA_Octet*&, bool);

#endif
